function testbench_FIR5_auto(SFxd3,DBG)
%Syntax: TESTBENCH_FIR5_AUTO(SFxd3,DBG)
%	SFxd3 defines width of signed fractional fixed point databus
%	in vector [N M x], where N is the total number of bits of the
%	external I/O bus from which M are fractional bits.
%	x defines an additional number of bits to extent N inside the SSG.
%	If DBG = 1, intermediate results are printed.
%	Needs 'FIR5.INP' in current directory to read input data from,
%	writes 'FIR5.OUT' with results.

global   c0 c1 c2 c3 c4 c5  i1 i2 i3 i4 i5

N = SFxd3(1);
M = SFxd3(2);
if ( N == M )
	error( 'SFxd leaves no room for a sign bit ...' );
end
NX = N + SFxd3(3);
fxd  = [N  M];
SFxd = [NX M];

% reset delayed inputs
i1 = 0;
i2 = 0;
i3 = 0;
i4 = 0;
i5 = 0;

fidi = fopen( 'FIR5.INP', 'r' );
if ( fidi == -1 )
	error( 'Cannot open file ''FIR5.INP'' ...' );
	beep;
end
fido = fopen( 'FIR5.OUT', 'w' );
fprintf( fido, 'MATLAB testbench output: %s\n', datestr(now) );
fprintf( fido, 'SFxd3 = [%d %d %d]\n', SFxd3 );
% simulate one SAMPLE-clock delay for each output
fprintf( fido, 'x"%s"\n', fixp2hex(0,fxd) );

% read constants from file
fprintf( 'Reading file ''FIR5.INP'' ...\n' );
c0 = lfReadVhdHex(fidi,fxd);
c1 = lfReadVhdHex(fidi,fxd);
c2 = lfReadVhdHex(fidi,fxd);
c3 = lfReadVhdHex(fidi,fxd);
c4 = lfReadVhdHex(fidi,fxd);
c5 = lfReadVhdHex(fidi,fxd);

i   = 1;
EOF = 0;
while ~EOF
	% read input value(s) from file
	[i0,EOF] = lfReadVhdHex(fidi,fxd);
	if ~EOF
		if DBG,  fprintf('\nPASS %d:\n', i);  end
		%=======  Call generated circuit function  ============
		[o5] = TB_FIR5_auto(SFxd,i0,DBG);
		% write output value(s) to file and to screen
		fprintf( fido, 'x"%s"\n', fixp2hex(o5,fxd) );
		fprintf( 'o5:  x"%s"   %#- 10g\n', fixp2hex(o5,fxd), o5 );
		i = i +1;
	end
end
fclose('all');
fprintf( 'written file ''FIR5.OUT''\n\n' );


%================================================================================
%=======   LOCAL FUNCTIONS   ==================================================== 
%================================================================================
function [outInt,EOF] = lfReadVhdHex(fid,fxd)
  inLine = fgetl(fid);
  % skip blank lines
  while ( ischar(inLine) && (length(inLine) < 4) )
    inLine = fgetl(fid);
  end
  % skip if not hex input line
  while ( ischar(inLine) && (inLine(1) ~= 'x') )
    inLine = fgetl(fid);
  end
  if ischar(inLine)
    dQuotePos = find( inLine == '"' );
    if ( length(dQuotePos) ~= 2 )
      error( 'INP FILE: %s, incorrect hex input format ...', inLine );
    end
    ix     = dQuotePos + [ 1 -1 ];
    outInt = hex2fixp( inLine(ix(1):ix(2)),fxd );
    EOF    = 0;
  else
    outInt = -1;
    EOF    =  1;
  end

%[EOF]

