function result = CIRC2cir(CIRC_Data,cirFilename)

% (c) H.J. Lincklaen Arriens, 
%     Delft University of Technology, October 2005

if exist('cirFilename')
	fid = fopen(cirFilename,'w');
	if ( fid == -1 )
		error( 'Cannot create file ...' );
	end
else
	fid = 1;
end
fprintf( fid, '%% Circuit description translated by CIRC2cir.m on %s\n\n', ...
															 datestr(clock) );

iocDef   = CIRC_Data.iocDef;
iconnTbl = CIRC_Data.iconnTbl;
xconnTbl = CIRC_Data.xconnTbl;
opTypes  = CIRC_Data.opTypes;
opNames  = CIRC_Data.opNames; 
inpNames = CIRC_Data.inpNames;
outNames = CIRC_Data.outNames;
dlydInps = CIRC_Data.dlydInps;

InOuts    = groupIOs(CIRC_Data);

constCoeffs = InOuts.constCoeffs;
extInps     = InOuts.extInps;
ssgInps     = InOuts.ssgInps;
ssgOutps    = InOuts.ssgOutps;
extOutps    = InOuts.extOutps;
dlydInpsTbl = InOuts.dlydInpsTbl;

nCoeffs   = size(constCoeffs,1); 
nExtInps  = size(extInps,1);     
nSsgInps  = size(ssgInps,1);     
nSsgOutps = size(ssgOutps,1);
nExtOutps = size(extOutps,1);  
nDlydInps = size(dlydInps,1);


intOutps = ssgOutps;
fprintf( fid, '%% external input(s) : ' );
for i = 1:nExtInps-1
	fprintf( fid, '''%s'', ', char(extInps(i)) );
end
fprintf( fid, '''%s''.\n', char(extInps(end)) ); 
fprintf( fid, '%% external output(s): ' );
for i = 1:nExtOutps-1
	fprintf( fid, '''%s'', ', char(extOutps(i)) );
	intOutps = intOutps( ~strcmp(intOutps,extOutps(1)) );
end
fprintf( fid, '''%s''.\n\n', char(extOutps(end)) ); 
intOutps = intOutps( ~strcmp(intOutps,extOutps(end)) );


fprintf( fid, '%% delayed input(s):\n' );
for i = 1:size(dlydInpsTbl,1)
	fprintf( fid, '%s = T%s;\n', char(dlydInpsTbl(i,1)), char(dlydInpsTbl(i,2)) );
end
fprintf( fid, '\n' );

fprintf( fid, '%% internal output(s):\n' );
for i = 1:length(intOutps)
	ix = find( strcmp(outNames,intOutps(i)) );
	opNo = find( iconnTbl(:,4,1) == ix );
	fprintf( fid, '%s = %s;\n', char(intOutps(i)), char(opNames(opNo)) );
end
fprintf( fid, '\n' );


fprintf( fid, '%% operations\n' );
operStr = '*+-';
for i = 1:size(iconnTbl,1)
	operChar = operStr( find(strcmp([{'mul'};{'add'};{'sub'}],opTypes(i,:))) );
	for iOp = 1:2
		opNo = iconnTbl(i,iOp+1,1);
		if ( opNo > 0 )
			opStr(iOp) = opNames(opNo);
		else
			opStr(iOp) = inpNames(-opNo);
		end
	end
	fprintf( fid, '%s = %s %s %s;\n', char(opNames(iconnTbl(i,1,1))), ...
									char(opStr(1)), operChar, char(opStr(2)) );
end
fprintf( fid, '\n' );

if any(any(iconnTbl(:,:,2)))
	fprintf( fid, '%% shift operations\n' );
	[row,clmn] = find( iconnTbl(:,:,2) ~= 0 );
	for i = 1:length(row)
		opNo = iconnTbl(row(i),clmn(i),1);
		if ( opNo > 0 )
			shOpStr = opNames(opNo);
		else
			shOpStr = inpNames(-opNo);
		end
		shiftVal = iconnTbl(row(i),clmn(i),2);
		if ( shiftVal > 0 )
			shiftDirStr = '>>';
		else
			shiftDirStr = '<<';
		end
		if ( clmn(i) == 1 )	% output = shift thisOp
			outOpStr = outNames(iconnTbl(row(i),4,1));	
		else
			outOpStr = opNames(opNo);
		end
		fprintf( fid, '%s = (%s %s %d);\n', ...
					char(outOpStr), char(shOpStr), shiftDirStr, abs(shiftVal) );
	end
end
fprintf( fid, '\n' );

fprintf( fid, '%% (C) Huib, 2005\n%% [EOF]\n' );

if ( fid ~= 1 )
	fclose(fid);
end
