function tFrames = asap(adjMat,delayVec)
%ASAP	Find the SSG using the ASAP	method
%	tFrames = ASAP(adjMat,delayVec) returns a two-column vector 'tFrames'
%	describing the SSG (Scheduled Sequencing Graph) of the operations 
%	when scheduled according to the ASAP (As Soon As Possible) method 
%	(no resource constraints).
%	The unscheduled flow has to be given in	'adjMat' with the clock delays 
%	per resource type (e.g. MUL, ALU) as integer number of clock cycles 
%	specified in 'delayVec'.
%
%	See also ALAP, FORCED, LISTSCHED, TOADJMAT, PARSE.  

% (c) H.J. Lincklaen Arriens, 
%     Delft University of Technology, November 2004


nNodes       = length(delayVec);
delayMat     = repmat(delayVec,1,nNodes) .* adjMat;

tFrames      = zeros(nNodes,2);
schedOps     = zeros(nNodes,1);
scheduleNow  = find( sum(delayMat) == 0 );
pendingNodes = any(scheduleNow);
t = 1;
while pendingNodes
	tFrames(scheduleNow,1)      = t;
	tFrames( (schedOps > 0), 2) = t;
	schedOps(scheduleNow)       = delayVec(scheduleNow);
	delayMat(:,scheduleNow)     = NaN;
	% adjust for time step
	pendingOps                  = find(schedOps > 0);
	schedOps(pendingOps)     = schedOps(pendingOps)   -1;
	delayMat(pendingOps,:)   = delayMat(pendingOps,:) -1;
	delayMat( delayMat < 0 ) = 0;
	% next time step
	t = t + 1;
	scheduleNow  = find( sum(delayMat) == 0 );
	pendingNodes = ( any(scheduleNow) | any(schedOps) );
end

