function axdrag2(action,hx,textHandles)
%AXDRAG2  Pan and zoom with mouse and simple keystrokes
%   Use this tool to move quickly around the data displayed in a 2-D plot.
%   Make sure the figure has focus, and then press any of the following
%   keys to zoom in or out. Clicking and dragging will pan the data.
%
%   Keys you can use are:
%   z, Z: zoom in, zoom out, in both dimensions
%   arrow keys (or pushed down left mouse button): pan the data
%   h: toggle help on/off
% 
%	Originally 'axdrag.m', written by Ned Gulley, March 2003
%
%
%	Adapted by	H.J. Lincklaen Arriens,
%				Delft University of Technology, December 2004
%
%	a.o. '+' and '-' for zooming, Help moved to schedGUI (helpwin)
%
%	my syntax:  AXDRAG2(action,hx,textHandles)
%		in which p.e. 'hx' is the handle of the 'gca' of the plot window
%		to work in, and 'textHandles' are the handles of the strings that 
%		should also be affected when zooming.

persistent started orgZoomFactor x0 dx h txtHandles orgFontSize nTimesZoomed 

if nargin >= 2
	h = hx;
	txtHandles = textHandles;
	if ~isempty(txtHandles)
		orgFontSize = get( txtHandles(1),'FontSize');
	end
end

% Use these variables to change the zoom and pan amounts
zoomFactor = 0.9;
panFactor  = 0.02;
    
switch action
    
	case 'initialize'
    	set(h,'ButtonDownFcn','axdrag2 start')
	    set(gcf,'KeyPressFcn','axdrag2 keypress')
    	set(gcf,'DoubleBuffer','on')
    	orgZoomFactor = zoomFactor;
    	nTimesZoomed = 0;
        started = 0;
    	axis auto;
    
	case 'start'
    	set(gcbf,'Units','pixel');
	    set(h,'Units','pixel');
    	set(gcbf,'WindowButtonMotionFcn','axdrag2 move')
	    set(gcbf,'WindowButtonUpFcn','axdrag2 stop')
    	currentPoint = get(gcbf,'CurrentPoint');
	    x0 = currentPoint;
        started = 1;
    	axdrag2 move
	
	case 'move'
    	currentPoint = get(gcbf,'CurrentPoint');
    	dx   = currentPoint - x0;
	    x0   = currentPoint;
    	ap   = get(h,'Position');
	    xLim = get(h,'XLim');
    	yLim = get(h,'YLim');
    	set(h,'XLim',xLim-(diff(xLim)*dx(1)/ap(3)), ...
									'YLim',yLim-(diff(yLim)*dx(2)/ap(4)));
    
	case 'stop'
    	set(gcbf,'WindowButtonMotionFcn','')
	    set(gcbf,'WindowButtonUpFcn','')
    	set(gcbf,'Units','normalized');
    	set(h,'Units','normalized');
    
	case 'keypress'
    	if started
            currChar = get(gcbf,'CurrentCharacter');
            if isempty(currChar) 
                return
            end
            switch currChar
                case 29					% RIGHT arrow key
                    xLim    = get(h,'XLim');
                    xLimNew = xLim + panFactor*diff(xLim);
                        set(h,'XLim',xLimNew)
            	case 28					% LEFT arrow key
                	xLim    = get(h,'XLim');
                    xLimNew = xLim - panFactor*diff(xLim);
            		set(h,'XLim',xLimNew)
            	case 31					% DOWN arrow key
                	yLim    = get(h,'YLim');
                	yLimNew = yLim - panFactor*diff(yLim);
                    set(h,'YLim',yLimNew)
    	    	case 30					% UP arrow key
            	    yLim    = get(h,'YLim');
                	yLimNew = yLim + panFactor*diff(yLim);
                	set(h,'YLim',yLimNew)
    	    	case {'+','-'}
            		if ( currChar == '+' )
            			nTimesZoomed = nTimesZoomed +1;
                    else
                		zoomFactor   = 1/zoomFactor;
                		nTimesZoomed = nTimesZoomed -1;
                    end
        	  		xLim    = get(h,'XLim');
            		yLim    = get(h,'YLim');
                	xLimNew = [0 zoomFactor*diff(xLim)] + xLim(1) + (1-zoomFactor)*diff(xLim)/2;
                    yLimNew = [0 zoomFactor*diff(yLim)] + yLim(1) + (1-zoomFactor)*diff(yLim)/2;
    				set(h,'XLim',xLimNew,'YLim',yLimNew)
        			% resize fonts
            		set(txtHandles, 'FontSize',orgFontSize/(orgZoomFactor^nTimesZoomed));
    	    	case {'*'}
                	ZoomFactor = orgZoomFactor;
            	   	nTimesZoomed = 0;
                	set(txtHandles, 'FontSize',orgFontSize);
                    axis auto;
            end
        end
end


