function decVal = hex2fixp(hexStr,fxd)
%HEX2FIXP	Convert a hex string given by fxd-bits into its signed decimal equivalent
%	decVal = HEX2FIXP(hexStr,fxd) converts 'hexStr' into a (possibly fractional)
%	signed decimal value in 'fxd' bits. 
%	'fxd' is supposed to be a two-element vector [N M], where N defines
%	the total number of available bits and M defines the number of bits
%	to be used at the right of the binary point.
%	An error occurs when 'decVal' cannot be represented in the given 'fxd' bits.
%	'hexStr' can be an array of strings.
%	Note: usually 'format long' will be necessary to represent the result 
%		  with enough decimals (avoid rounding to 'short' format).
%
%	See also FIXP2HEX, TOSFIXP, TOUFIXP.  

% (c) H.J. Lincklaen Arriens, 
%     Delft University of Technology, November 2004


if ( nargin ~= 2 )
	error( 'syntax: HEX2FIXP(hexStr,fxd)) ...' );
end
N = fxd(1);
M = fxd(2);
if ( N <= M )
	error( 'Incorrect values for parameter Fixed ...' );
end

maxVal  = 2^N;
fracFac = 2^M;
nValues = size(hexStr,1);
decVal  = zeros(nValues,1);

for i = 1:nValues
	testValue = hex2dec( hexStr(i,:) );
	if ( testValue > (maxVal-1) )
		error( 'HEX2FIXP: %s doesn''t fit in %d bits ...', hexStr(i,:), N );
	end 

	if ( testValue >= maxVal/2 )
		decVal(i) = ( testValue - maxVal ) / fracFac;
	else
		decVal(i) = testValue / fracFac;
	end
end

