function y = read_OUT(outFilename,fxd2,nOutputs,nFracDigits)
%READ_OUT	Read .OUT file and return formatted readable data
%	READ_OUT(outFilename,fxd2)
%	y = READ_OUT(outFilename,fxd2)
%	y = READ_OUT(outFilename,fxd2,nOutputs)
%	y = READ_OUT(outFilename,fxd2,nOutputs,nFracDigits)

% (c) H.J. Lincklaen Arriens, 
%     Delft University of Technology, November 2005


N = fxd2(1);
M = fxd2(2);

if ~strcmp(outFilename(end-3:end),'.OUT')
	outFilename = [ outFilename '.OUT' ];
end
fid = fopen( outFilename, 'r' );
if ( fid == -1 )
	error( 'Cannot find file ''%s'' ...', outFilename );
end
if ( nargin < 3 )
	nOutputs = 1;
end
nIntegerDigits = max(1,ceil(log10(2^(N-M-1)))) +1;  % for sign bit
if ( nargin < 4 )
	nFracDigits = 17 - nIntegerDigits;
end


% read complete .OUT file
i = 1;
inpLine = fgetl( fid );
while ( inpLine ~= -1 )
	if ( inpLine(1) == 'x' )
		inLines(i,:) = inpLine;
		i = i +1;
	end
	inpLine = fgetl( fid );
end
fclose( fid );

% test for correct number of bits: specified vs read
nHexDigits = ceil(N/4);
if ( (size(inLines,2)-3) ~= nHexDigits )
	error( 'Wrong fixed-point format ...' );
end

% add zero outputs in case some are missing
nLines = i -1;
nLeft  = rem(nLines,nOutputs);
if ( nLeft ~= 0 )
	zeroStr = [ 'x"' char('0' * ones(1,nHexDigits))	'"' ];
	nToAdd  = nOutputs-nLeft;
	for i = 1:nToAdd
		inLines(nLines+i,:) = zeroStr;
	end
	nLines = nLines + nToAdd;
end

nRows = nLines/nOutputs;
str1  = [ 'fprintf( ''%s = %' ];
str2  = int2str(nIntegerDigits+1+nFracDigits);
str3  = int2str(nFracDigits);
str4  = [ 'f    '', inLines(ix,:), yTmp );' ];
evalStr = [ str1 str2 '.' str3 str4 ];
ix = 1;
y  = zeros(nRows,nOutputs);
for iRow = 1:nRows
	fprintf( '%3d:  ', iRow );
	for iOut = 1:nOutputs
		yTmp = hex2fixp(inLines(ix,3:end-1),[N M]);
		eval( evalStr );
		y(iRow,iOut) = yTmp;
	    ix = ix +1;
	 end
	 fprintf( '\n' );
end
fprintf( '\n' );
