function SFixValue = toSFixp(value,SFxd,fMode)
%TOSFIXP	Converts a signed fractional value to fit in SFxd bits.
%	SFixValue = TOSFIXP(value,SFxd) truncates (= towards minus infinity),
%	the	signed fractional 'value' to fit in the given 'SFxd' bits and 
%	returns it in 'SFixValue'.
%	'SFxd' is supposed to be a two-element vector [N M], where N defines
%	the total number of available bits and M defines the number of bits
%	to the right of the binary point. 
%	'value' can be a vector of signed fractional decimals. 
%
%	SFixValue = TOSFIXP(value,SFxd,'round') can be used to do a rounding
%	operation instead of just a truncation.
%
%	See also FIXP2HEX, HEX2FIXP, TOUFIXP.  

% (c) H.J. Lincklaen Arriens, 
%     Delft University of Technology, November 2004

if ~exist('fMode')
	fMode = 'trunc';
end

N = SFxd(1);
M = SFxd(2);
if ( N <= M )
	error( 'Incorrect values for SIGNED parameter ...' );
end

maxPos  =  2^(N-1) -1;
maxNeg  = -2^(N-1);
fracFac =  2^M;

nValues = length(value);
for i = 1:nValues
	testValue = value(i) * fracFac;
	if ( (testValue > maxPos) || (testValue < maxNeg) )
		error( 'SFIX ERROR: "%g" doesn''t fit in %d (signed) integer bits ...', ...
																		value(i), N-M );
	end
end
if strcmp(fMode,'round')
	SFixValue = round( value * fracFac ) / fracFac;
else
	% round towards minus infinity ('trunc')
	SFixValue = floor( value * fracFac ) / fracFac;
end
