function UFixValue = toUFixp(value,UFxd)
%TOUFIXP	Converts an unsigned fractional value to fit in UFxd bits.
%	UFixValue = TOUFIXP(value,UFxd) truncates, if necessary, the
%	unsigned fractional 'value' to fit in the given 'UFxd' bits and 
%	returns it in 'UFixValue'.
%	'UFxd' is supposed to be a two-element vector [N M], where N defines
%	the total number of available bits and M defines the number of bits
%	to the right of the binary point.
%	'value' can be a vector of unsigned fractional decimals. 
%
%	See also FIXP2HEX, HEX2FIXP, TOSFIXP.  

% (c) H.J. Lincklaen Arriens, 
%     Delft University of Technology, November 2004


N = UFxd(1);
M = UFxd(2);
if ( N < M )
	error( 'Incorrect values for UNSIGNED parameter ...' );
end

maxPos  =  2^N -1;
fracFac =  2^M;

nValues = length(value);
for i = 1:nValues
	testValue = value(i) * fracFac;
	if ( testValue < 0 )
		error( 'UFIX ERROR: %g not Unsigned Positive ...', value(i) );
    end
	if ( testValue > maxPos )
		error( 'UFIX ERROR: "%g" doesn''t fit in %d integer bits ...', value(i), N-M );
	end
end
UFixValue = round( value * fracFac ) / fracFac;
